<?php
class Simplr_Onlinerefunds_Model_Onlinerefund_Api extends Mage_Sales_Model_Api_Resource
{
    const ONE_CENT = 0.01;

    /**
     * Initialize attributes map for order (will be used on Simplr)
     */
    public function __construct()
    {
        $this->_attributesMap = array(
            'order' => array('order_id' => 'entity_id'),
        );
    }

    /**
     * Router for SOAP requests from Simplr
     *
     * @param string $jsonParams
     * @return string
     * @throws Mage_Api_Exception
     */
    public function soapcall($jsonParams)
    {
        try {
            $params = json_decode($jsonParams, True);

            $method = $params['method'];
            $methodArgs = $params['args'];

            $refClass = new ReflectionClass(get_class($this));
            $refMethod = $refClass->getMethod($method);

            $result = $refMethod->invokeArgs($this, $methodArgs);

            return json_encode($result);
        } catch (ReflectionException $e) {
            return $this->_fault('soap_no_method_error');
        } catch (JsonException $e) {
            return $this->_fault('soap_json_error', $e->getMessage());
        } catch (Exception $e) {
            return $this->_fault('soap_error', $e->getMessage());
        }
    }

    /**
     * Returns extension version
     *
     * @return string
     */
    public function version()
    {
        return (string) Mage::getConfig()->getNode()->modules->Simplr_Onlinerefunds->version;
    }

    /**
     * Returns refund total amounts
     * @param  string $orderIncrementId
     * @param  array $creditmemoData
     * @param  number|null $refundToStoreCreditAmount
     * @return array
     * @throws Mage_Api_Exception
     */
    public function calculate($orderIncrementId, $creditmemoData = [], $refundToStoreCreditAmount = null)
    {
        $order = $this->_loadOrder($orderIncrementId);
        $invoice = $order->getInvoiceCollection()->getItemByColumnValue('entity_id', $creditmemoData['invoiceId']);
        $creditMemo = $this->_prepareCreditMemo($order, $invoice, $creditmemoData, $refundToStoreCreditAmount);
        return $this->_generateTotalsCreditMemo($creditMemo);
    }

    public function create($orderIncrementId, $creditmemoData = [], $comment = null, $notifyCustomer = false,
                           $includeComment = false, $refundToStoreCreditAmount = null)
    {
        $order = $this->_loadOrder($orderIncrementId);
        $invoice = $order->getInvoiceCollection()->getItemByColumnValue('entity_id', $creditmemoData['invoiceId']);
        $creditmemo = $this->_prepareCreditMemo($order, $invoice, $creditmemoData, $refundToStoreCreditAmount);

        try {
            // Perform actual refund
            $creditmemo->register();

            // Add Simplr comment to creditmemo and order
            $orderComment = 'Refunded by Simplr' . ($comment ? ': ' . $comment : '');
            $creditmemo->addComment($orderComment);
            $order->addStatusHistoryComment($orderComment);

            // Save all objects altogether
            Mage::getModel('core/resource_transaction')
                ->addObject($creditmemo)
                ->addObject($order)
                ->addObject($invoice)
                ->save();

            // Saving order second time helps solve problems with wrong status
            // Need to reload order from DB to have correct qty's for recalculation
            $order = $this->_loadOrder($order->getIncrementId());
            // Mark order as dirty so it will call beforeSave hooks
            $order->setDataChanges(true);
            $order->save();

            // send email notification
            $creditmemo->sendEmail($notifyCustomer, ($includeComment ? $comment : ''));
        } catch (Mage_Core_Exception $e) {
            $this->_fault('data_invalid', $e->getMessage());
        }

        // Reload creditmemo and get totals data
        $creditmemo = Mage::getModel('sales/order_creditmemo')->load($creditmemo->getIncrementId(), 'increment_id');
        $creditMemoData = $this->_getAttributes($creditmemo, 'creditmemo');
        $creditMemoData['totals_info'] = $this->_generateTotalsCreditMemo($creditmemo);

        return $creditMemoData;
    }

    public function refundable($orderIncrementId)
    {
        /** @var $order Mage_Sales_Model_Order */
        $order = Mage::getModel('sales/order')->load($orderIncrementId, 'increment_id');

        $invoicesCollection = Mage::getResourceModel('sales/order_invoice_collection')
            ->addFieldToFilter('order_id', $order->getId());

        $resultInvoices = array();
        foreach ($invoicesCollection as $invoice) {
            $invoiceAttrs = $this->_getAttributes($invoice, 'invoice');

            $refundableShipping = $invoiceAttrs['shipping_amount'];

            $invoiceItemsCollection = $invoice->getAllItems();
            $invoiceItems = array();
            $refundableItems = array();
            foreach ($invoiceItemsCollection as $invoiceItem) {
                $invoiceItemAttrs = $this->_getAttributes($invoiceItem, 'invoice_item');
                $orderItem = $invoiceItem->getOrderItem();

                if ($orderItem->isDummy()) {
                    continue;
                }

                $invoiceItemAttrs['order_item'] = $this->_getAttributes($orderItem, 'order_item');
                $invoiceItems[] = $invoiceItemAttrs;
                $refundableItems[$invoiceItemAttrs['order_item_id']] = $invoiceItemAttrs['qty'];
            }

            $invoiceCreditMemosCollection = Mage::getResourceModel('sales/order_creditmemo_collection')
                ->addFieldToFilter('invoice_id', $invoice->getId());
            $creditMemos = array();
            foreach ($invoiceCreditMemosCollection as $creditMemo) {
                if ($creditMemo->getState() == Mage_Sales_Model_Order_Creditmemo::STATE_CANCELED) {
                    continue;
                }

                $creditMemoAttrs = $this->_getAttributes($creditMemo, 'creditmemo');

                $refundableShipping -= $creditMemoAttrs['shipping_amount'];

                $creditMemoItemsCollection = $creditMemo->getAllItems();
                $creditMemoItems = array();
                foreach ($creditMemoItemsCollection as $creditMemoItem) {
                    $creditMemoItemAttrs = $this->_getAttributes($creditMemoItem, 'creditmemo_item');
                    $creditMemoItems[] = $creditMemoItemAttrs;

                    if (isset($refundableItems[$creditMemoItemAttrs['order_item_id']])) {
                        $refundableItems[$creditMemoItemAttrs['order_item_id']] -= $creditMemoItemAttrs['qty'];
                    }
                }

                $creditMemoAttrs['totals_info'] = $this->_generateTotalsCreditMemo($creditMemo);
                $creditMemoAttrs['items'] = $creditMemoItems;
                $creditMemos[] = $creditMemoAttrs;
            }

            // Add refundable quantity to items list
            for ($i = 0; $i < count($invoiceItems); $i++) {
                $invoiceItems[$i]['refundable_qty'] = $refundableItems[$invoiceItems[$i]['order_item_id']];
            }

            $invoiceAttrs['invoice_id'] = $invoice->getId();
            $invoiceAttrs['credit_memos'] = $creditMemos;
            $invoiceAttrs['items'] = $invoiceItems;
            $invoiceAttrs['refundable_items'] = $refundableItems;
            $invoiceAttrs['refundable_shipping'] = $refundableShipping;
            $resultInvoices[] = $invoiceAttrs;
        }

        return $resultInvoices;
    }

    /**
     * Copied from Mage_Sales_Model_Order_Creditmemo_Api
     * _prepareCreateData returned format:
     * array(
     *   'qtys' => array(
     *     '<order_item_id>' => <qty>
     *   )
     *   ... everything else
     * );
     * @param  array $data
     * @return array
     */
    protected function _prepareCreateData($data)
    {
        $data = isset($data) ? $data : array();

        if (isset($data['qtys']) && count($data['qtys'])) {
            $qtysArray = array();
            foreach ($data['qtys'] as $qKey => $qVal) {
                // Save backward compatibility
                if (is_array($qVal)) {
                    if (isset($qVal['order_item_id']) && isset($qVal['qty'])) {
                        $qtysArray[$qVal['order_item_id']] = $qVal['qty'];
                    }
                } else {
                    $qtysArray[$qKey] = $qVal;
                }
            }
            $data['qtys'] = $qtysArray;
        }
        return $data;
    }

    public function cancelorderpartial($orderIncrementId, $qtys, $comment = null, $notifyCustomer = false, $includeComment = false)
    {
        /** @var $order Mage_Sales_Model_Order */
        $order = $this->_loadOrder($orderIncrementId);

        $nonCancellableOrderStates = array(
            Mage_Sales_Model_Order::STATE_COMPLETE,
            Mage_Sales_Model_Order::STATE_CANCELED,
            Mage_Sales_Model_Order::STATE_CLOSED,
            Mage_Sales_Model_Order::STATE_HOLDED
        );
        if (in_array($order->getState(), $nonCancellableOrderStates)) {
            $this->_fault('data_invalid', 'Order state is invalid to be cancelled');
        }

        if (!is_array($qtys)) {
            $this->_fault('data_invalid', 'qtys should be an array');
        }

        $needToFullCancel = true;

        // First loop to check qty's are valid
        /** @var $item Mage_Sales_Model_Order_Item */
        foreach ($order->getAllItems() as $item) {
            $itemId = $item->getId();

            $qtyToCancel = $this->_getQtyByItemId($qtys, $itemId);

            if ($qtyToCancel == 0) {
                continue;
            }

            $qtyCancellable = $item->getQtyOrdered() - $item->getQtyCanceled() - max($item->getQtyInvoiced(), $item->getQtyShipped());

            if ($qtyToCancel > $qtyCancellable) {
                $this->_fault('data_invalid', 'You requested too much of ' . $itemId . ' to cancel');
            } else {
                if (($item->getQtyCanceled() + $qtyToCancel) != $item->getQtyOrdered()) {
                    $needToFullCancel = false;
                }
            }
        }

        $orderNotice = ($needToFullCancel ? 'Cancelled' : 'Partially cancelled');
        $orderNotice .= ' by Simplr';
        $orderNotice .= $comment ? ': ' . $comment : '';

        if ($needToFullCancel) {
            $order->cancel();
            $order->save();

            foreach ($order->getAllItems() as $item) {
                $item->setQtyCanceled($item->getQtyOrdered());
                $this->_updateItemCancelledTaxes($item);

                $item->save();
            }
        } else {
            // Do partial cancel
            $orderNotice .= "\nItems:";

            foreach ($order->getAllItems() as $item) {
                $itemId = $item->getId();

                $qtyToCancel = $this->_getQtyByItemId($qtys, $itemId);

                if ($qtyToCancel == 0) {
                    continue;
                }

                $item->setQtyCanceled($item->getQtyCanceled() + $qtyToCancel);
                $this->_updateItemCancelledTaxes($item);

                $item->save();

                Mage::dispatchEvent('sales_order_item_cancel', array('item' => $item));

                $itemName = $item->getName();
                $orderNotice .= "\n${itemName}: ${qtyToCancel}";
            }
        }

        $order->addStatusHistoryComment($orderNotice)->save();

        return $order;
    }

    /**
     * Copied from Mage_Sales_Model_Order_Item#cancel
     * @param Mage_Sales_Model_Order_Item $item
     */
    private function _updateItemCancelledTaxes(Mage_Sales_Model_Order_Item $item)
    {
        $item->setTaxCanceled(
            $item->getTaxCanceled() + $item->getBaseTaxAmount() * $item->getQtyCanceled() / $item->getQtyOrdered()
        );
        $item->setHiddenTaxCanceled(
            $item->getHiddenTaxCanceled() + $item->getHiddenTaxAmount() * $item->getQtyCanceled() / $item->getQtyOrdered()
        );
    }

    /**
     * Returns qty of items that requested to be cancelled of specific type
     * @param array $qtys
     * @param string $itemId
     * @return int
     */
    private function _getQtyByItemId($qtys, $itemId)
    {
        foreach ($qtys as $qty) {
            if (strval($qty['order_item_id']) == strval($itemId)) {
                return $qty['qty'];
            }
        }

        return 0;
    }

    /**
     * Retrieve list of orders. Filtration could be applied
     * Copied from Mage_Sales_Model_Order_Api
     *
     * @param null|object|array $filters
     * @return array
     * @throws Mage_Api_Exception
     */
    public function listorders($filters = null)
    {
        $orders = array();

        //TODO: add full name logic
        $billingAliasName = 'billing_o_a';
        $shippingAliasName = 'shipping_o_a';

        /** @var $orderCollection Mage_Sales_Model_Mysql4_Order_Collection */
        $orderCollection = Mage::getModel("sales/order")->getCollection();
        $billingFirstnameField = "$billingAliasName.firstname";
        $billingLastnameField = "$billingAliasName.lastname";
        $shippingFirstnameField = "$shippingAliasName.firstname";
        $shippingLastnameField = "$shippingAliasName.lastname";
        $orderCollection->addAttributeToSelect('*')
            ->addAddressFields()
            ->addExpressionFieldToSelect('billing_firstname', "{{billing_firstname}}",
                array('billing_firstname' => $billingFirstnameField))
            ->addExpressionFieldToSelect('billing_lastname', "{{billing_lastname}}",
                array('billing_lastname' => $billingLastnameField))
            ->addExpressionFieldToSelect('shipping_firstname', "{{shipping_firstname}}",
                array('shipping_firstname' => $shippingFirstnameField))
            ->addExpressionFieldToSelect('shipping_lastname', "{{shipping_lastname}}",
                array('shipping_lastname' => $shippingLastnameField))
            ->addExpressionFieldToSelect('billing_name', "CONCAT({{billing_firstname}}, ' ', {{billing_lastname}})",
                array('billing_firstname' => $billingFirstnameField, 'billing_lastname' => $billingLastnameField))
            ->addExpressionFieldToSelect('shipping_name', 'CONCAT({{shipping_firstname}}, " ", {{shipping_lastname}})',
                array('shipping_firstname' => $shippingFirstnameField, 'shipping_lastname' => $shippingLastnameField)
            );

        /** @var $apiHelper Mage_Api_Helper_Data */
        $apiHelper = Mage::helper('api');
        $filters = $apiHelper->parseFilters($filters, $this->_attributesMap['order']);
        try {
            foreach ($filters as $field => $value) {
                $orderCollection->addFieldToFilter($field, $value);
            }
        } catch (Mage_Core_Exception $e) {
            $this->_fault('filters_invalid', $e->getMessage());
        }
        foreach ($orderCollection as $order) {
            $orders[] = $this->_getAttributes($order, 'order');
        }
        return $orders;
    }

    /**
     * Retrieve full order information
     * Copied from Mage_Sales_Model_Order_Api
     * Added logic to load related shipments
     *
     * @param string $orderIncrementId
     * @return Mage_Sales_Model_Api_Resource
     * @throws Mage_Api_Exception
     */
    public function orderinfo($orderIncrementId)
    {
        $order = $this->_loadOrder($orderIncrementId);

        if ($order->getGiftMessageId() > 0) {
            $order->setGiftMessage(
                Mage::getSingleton('giftmessage/message')->load($order->getGiftMessageId())->getMessage()
            );
        }

        $result = $this->_getAttributes($order, 'order');

        $result['shipping_address'] = $this->_getAttributes($order->getShippingAddress(), 'order_address');
        $result['billing_address']  = $this->_getAttributes($order->getBillingAddress(), 'order_address');
        $result['items'] = array();

        foreach ($order->getAllItems() as $item) {
            if ($item->getGiftMessageId() > 0) {
                $item->setGiftMessage(
                    Mage::getSingleton('giftmessage/message')->load($item->getGiftMessageId())->getMessage()
                );
            }

            $itemAttr = $this->_getAttributes($item, 'order_item');
            $itemAttr['rma_status_array'] = array();

            if (method_exists($item, 'getReturnLabelUrl')) {
                $itemAttr['rma_status_array']['label_url'] = $item->getReturnLabelUrl();
            }

            if (method_exists($item, 'getRMANumber')) {
                $itemAttr['rma_status_array']['rma_number'] = $item->getRMANumber();
            }

            if (method_exists($item, 'getReturnPolicyValues')) {
                $itemAttr['return_policy_values_array'] = $item->getReturnPolicyValues();
            }

            $result['items'][] = $itemAttr;
        }

        $result['payment'] = $this->_getAttributes($order->getPayment(), 'order_payment');

        $result['status_history'] = array();
        foreach ($order->getAllStatusHistory() as $history) {
            $result['status_history'][] = $this->_getAttributes($history, 'order_status_history');
        }

        // Add shipments to order info
        $shipments = array();
        $shipmentCollection = $order->getShipmentsCollection();

        foreach ($shipmentCollection as $shipment) {
            $shipmentsAttributes = $this->_getAttributes($shipment, 'shipment');

            $shipmentsAttributes['items'] = array();
            foreach ($shipment->getAllItems() as $item) {
                $shipmentsAttributes['items'][] = $this->_getAttributes($item, 'shipment_item');
            }

            $shipmentsAttributes['tracks'] = array();
            foreach ($shipment->getAllTracks() as $track) {
                $shipmentsAttributes['tracks'][] = $this->_getAttributes($track, 'shipment_track');
            }

            $shipments[] = $shipmentsAttributes;
        }
        $result['order_shipment'] = $shipments;


        return $result;
    }

    /**
     * Hold order
     * Copied from Mage_Sales_Model_Order_Api
     *
     * @param string $orderIncrementId
     * @return boolean
     * @throws Mage_Api_Exception
     */
    public function holdorder($orderIncrementId)
    {
        $order = $this->_loadOrder($orderIncrementId);

        try {
            $order->hold();
            $order->save();
        } catch (Mage_Core_Exception $e) {
            $this->_fault('status_not_changed', $e->getMessage());
        }

        return true;
    }

    /**
     * Unhold order
     * Copied from Mage_Sales_Model_Order_Api
     *
     * @param string $orderIncrementId
     * @return boolean
     * @throws Mage_Api_Exception
     */
    public function unholdorder($orderIncrementId)
    {
        $order = $this->_loadOrder($orderIncrementId);

        try {
            $order->unhold();
            $order->save();
        } catch (Mage_Core_Exception $e) {
            $this->_fault('status_not_changed', $e->getMessage());
        }

        return true;
    }

    /**
     * Cancel order
     * Copied from Mage_Sales_Model_Order_Api
     *
     * @param string $orderIncrementId
     * @return boolean
     * @throws Mage_Api_Exception
     */
    public function cancelorder($orderIncrementId)
    {
        $order = $this->_loadOrder($orderIncrementId);

        if (Mage_Sales_Model_Order::STATE_CANCELED == $order->getState()) {
            $this->_fault('status_not_changed');
        }
        try {
            $order->cancel();
            $order->save();
        } catch (Mage_Core_Exception $e) {
            $this->_fault('status_not_changed', $e->getMessage());
        }
        if (Mage_Sales_Model_Order::STATE_CANCELED != $order->getState()) {
            $this->_fault('status_not_changed');
        }
        return true;
    }

    /**
     * Checks that Enterprise_CustomerBalance module is available
     * @return boolean
     */
    public function checkcustomercreditsupport()
    {
        return is_object(Mage::getConfig()->getNode('modules/Enterprise_CustomerBalance'));
    }

    /**
     * Checks that custom methods available in order, customer and item classes
     * @return array
     */
    public function checkcustomcustomercreditsupport()
    {
        $customerModel = Mage::getModel("customer/customer");
        $itemModel = Mage::getModel("sales/order_item");

        $details = array(
            'issueStoreCreditSupported' => method_exists($customerModel, 'issueStoreCredit'),
            'getReturnWebsiteValues' => method_exists($customerModel, 'getReturnWebsiteValues'),
            'updateReturnPolicySupported' => method_exists($itemModel, 'updateReturnPolicy'),
            'getReturnPolicyValues' => method_exists($itemModel, 'getReturnPolicyValues')
        );

        $atLeastOneItemIsFalse = in_array(false, $details, true);
        return array(
            'details' => $details,
            'customerCreditSupported' => !$atLeastOneItemIsFalse
        );
    }

    /**
     * Issues store credit to customer
     * @param string $orderIncrementId
     * @param float $amount
     * @param string $website
     * @param string $comment
     * @return boolean
     * @throws Mage_Api_Exception
     */
    public function issuestorecredit($orderIncrementId, $amount, $website, $comment)
    {
        $order = $this->_loadOrder($orderIncrementId);
        $customer = $this->_loadCustomerByOrder($order);

        if (!method_exists($customer, 'issueStoreCredit')) {
            $this->_fault('method_not_exists', 'issueStoreCredit');
        }

        return $customer->issueStoreCredit($amount, $website, $comment);
    }


    /**
     * Get list of websites
     * @param string $orderIncrementId
     * @return array
     * @throws Mage_Api_Exception
     */
    public function getreturnwebsitevalues($orderIncrementId)
    {
        $customer = $this->_loadCustomerByOrderId($orderIncrementId);

        if (!method_exists($customer, 'getReturnWebsiteValues')) {
            $this->_fault('method_not_exists', 'getReturnWebsiteValues');
        }

        return $customer->getReturnWebsiteValues();
    }


    /**
     * Update return policy for OrderItem
     * @param string $orderIncrementId
     * @param string $itemId
     * @param string $returnPolicyValue
     * @return boolean; true if success, false if fails
     * @throws Mage_Api_Exception
     */
    public function updatereturnpolicy($orderIncrementId, $itemId, $returnPolicyValue)
    {
        $order = $this->_loadOrder($orderIncrementId);

        $orderItem = $order->getItemById($itemId);
        if (!$orderItem) {
            $this->_fault('item_not_exist');
        }

        if (!method_exists($orderItem, 'updateReturnPolicy')) {
            $this->_fault('method_not_exists', 'updateReturnPolicy');
        }

        return $orderItem->updateReturnPolicy($returnPolicyValue);
    }


    /**
     * Initialize and return Creditmemo with data
     * @param mixed $order
     * @param mixed $invoice
     * @param array $creditmemoData
     * @param number | null $refundToStoreCreditAmount
     * @return Mage_Sales_Model_Order_Creditmemo
     * @throws Mage_Api_Exception
     */
    protected function _prepareCreditMemo($order, $invoice, $creditmemoData, $refundToStoreCreditAmount)
    {
        if (!$order->canCreditmemo()) {
            $this->_fault('cannot_create_creditmemo');
        }

        if (!$invoice || !$invoice->getId()) {
            $this->_fault('invoice_not_exists');
        }

        if ($invoice->getOrder()->getId() != $order->getId()) {
            $this->_fault('cannot_create_creditmemo');
        }

        $creditmemoData = $this->_prepareCreateData($creditmemoData);

        /** @var $service Mage_Sales_Model_Service_Order */
        $service = Mage::getModel('sales/service_order', $order);
        /** @var $creditmemo Mage_Sales_Model_Order_Creditmemo */
        $creditmemo = $service->prepareInvoiceCreditmemo($invoice, $creditmemoData);

        if ($order->getPayment()->getMethod() == "free") {
            $creditmemo->setOfflineRequested(true);
        }

        // refund to Store Credit
        if ($refundToStoreCreditAmount) {
            // check if refund to Store Credit is available
            if ($order->getCustomerIsGuest()) {
                $this->_fault('cannot_refund_to_storecredit');
            }
            $refundToStoreCreditAmount = max(
                0,
                min($creditmemo->getBaseCustomerBalanceReturnMax(), $refundToStoreCreditAmount)
            );
            if ($refundToStoreCreditAmount) {
                $refundToStoreCreditAmount = $creditmemo->getStore()->roundPrice($refundToStoreCreditAmount);
                $creditmemo->setBaseCustomerBalanceTotalRefunded($refundToStoreCreditAmount);
                $refundToStoreCreditAmount = $creditmemo->getStore()->roundPrice(
                    $refundToStoreCreditAmount*$order->getStoreToOrderRate()
                );
                // this field can be used by customer balance observer
                $creditmemo->setBsCustomerBalTotalRefunded($refundToStoreCreditAmount);
                // setting flag to make actual refund to customer balance after credit memo save
                $creditmemo->setCustomerBalanceRefundFlag(true);
            }
        }

        $invoiceMaxRefundable = $invoice->getBaseGrandTotal() - $invoice->getBaseTotalRefunded();

        // Super hack to limit credit memo numbers to invoice ones
        // Without this we can get over invoice price by 1 cent and payment will be rejected
        if ($creditmemo->getBaseGrandTotal() > $invoiceMaxRefundable) {
            $creditmemo->setBaseGrandTotal($invoiceMaxRefundable);
            $creditmemo->setGrandTotal($invoiceMaxRefundable);
        }

        // Inverse to above - if we off by 1 cent down, round up
        if (($invoiceMaxRefundable - $creditmemo->getBaseGrandTotal()) <= self::ONE_CENT) {
            $creditmemo->setBaseGrandTotal($invoiceMaxRefundable);
            $creditmemo->setGrandTotal($invoiceMaxRefundable);
        }

        return $creditmemo;
    }


    /**
     * Generate and return total info from Creditmemo
     * @param mixed $creditMemo
     * @return array
     */
    protected function _generateTotalsCreditMemo($creditMemo) {
        $result = array(
            "subtotal" => $creditMemo->getSubtotal(),
            "discount" => $creditMemo->getDiscountAmount(),
            "tax" => $creditMemo->getTaxAmount(),
            "shipping" => $creditMemo->getBaseShippingAmount(),
            "total" => $creditMemo->getGrandTotal(),
            "custom" => array()
        );

        $defaultValues = array("subtotal", "discount", "tax", "shipping", "grand_total");

        foreach ($creditMemo->getConfig()->getTotalModels() as $model) {
            $code = $model->getCode();
            if (in_array($code, $defaultValues)) {
                continue;
            }

            $amount = $creditMemo->getData($code);

            if (is_numeric($amount)) {
                $result["custom"][$code] = $amount;
            }
        }

        return $result;
    }

    /**
     * Initialize basic order model
     * Copied from Mage_Sales_Model_Order_Api
     *
     * @param mixed $orderIncrementId
     * @return Mage_Sales_Model_Order
     * @throws Mage_Api_Exception
     */
    protected function _loadOrder($orderIncrementId)
    {
        $order = Mage::getModel('sales/order');

        /* @var $order Mage_Sales_Model_Order */
        $order->loadByIncrementId($orderIncrementId);

        if (!$order->getId()) {
            $this->_fault('order_not_exists');
        }

        return $order;
    }

    /**
     * Load basic customer model by $orderId
     * @param string $orderIncrementId
     * @return mixed
     * @throws Mage_Api_Exception
     */
    protected function _loadCustomerByOrderId($orderIncrementId)
    {
        $order = $this->_loadOrder($orderIncrementId);
        return $this->_loadCustomerByOrder($order);
    }

    /**
     * Load basic customer model by $order
     * @param Mage_Sales_Model_Order $order
     * @return mixed
     * @throws Mage_Api_Exception
     */
    protected function _loadCustomerByOrder($order)
    {
        $customerId = $order->getCustomerId();
        $customer = Mage::getModel('customer/customer')->load($customerId);

        /* @var $customer Mage_Customer_Model_Customer */
        if (!$customer->getId()) {
            $this->_fault('customer_not_exists');
        }

        return $customer;
    }
}
